@@ -21,16 +21,18 @@
  * Modified by the GTK+ Team and others 1997-2001.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
+#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
+#error "Only <gtk/gtk.h> can be included directly."
+#endif
+
 #ifndef __GTK_BUTTON_H__
 #define __GTK_BUTTON_H__
 
 
-#include <gdk/gdk.h>
 #include <gtk/gtkbin.h>
-#include <gtk/gtkenums.h>
 #include <gtk/gtkimage.h>
 
 
@@ -50,21 +52,21 @@ struct _GtkButton
 {
   GtkBin bin;
 
-  GdkWindow *event_window;
+  GdkWindow *GSEAL (event_window);
 
-  gchar *label_text;
+  gchar *GSEAL (label_text);
 
-  guint activate_timeout;
+  guint GSEAL (activate_timeout);
 
-  guint constructed : 1;
-  guint in_button : 1;
-  guint button_down : 1;
-  guint relief : 2;
-  guint use_underline : 1;
-  guint use_stock : 1;
-  guint depressed : 1;
-  guint depress_on_activate : 1;
-  guint focus_on_click : 1;
+  guint GSEAL (constructed) : 1;
+  guint GSEAL (in_button) : 1;
+  guint GSEAL (button_down) : 1;
+  guint GSEAL (relief) : 2;
+  guint GSEAL (use_underline) : 1;
+  guint GSEAL (use_stock) : 1;
+  guint GSEAL (depressed) : 1;
+  guint GSEAL (depress_on_activate) : 1;
+  guint GSEAL (focus_on_click) : 1;
 };
 
 struct _GtkButtonClass
@@ -91,11 +93,15 @@ GtkWidget*     gtk_button_new               (void);
 GtkWidget*     gtk_button_new_with_label    (const gchar    *label);
 GtkWidget*     gtk_button_new_from_stock    (const gchar    *stock_id);
 GtkWidget*     gtk_button_new_with_mnemonic (const gchar    *label);
+#ifndef GTK_DISABLE_DEPRECATED
 void           gtk_button_pressed           (GtkButton      *button);
 void           gtk_button_released          (GtkButton      *button);
+#endif
 void           gtk_button_clicked           (GtkButton      *button);
+#ifndef GTK_DISABLE_DEPRECATED
 void           gtk_button_enter             (GtkButton      *button);
 void           gtk_button_leave             (GtkButton      *button);
+#endif
 
 void                  gtk_button_set_relief         (GtkButton      *button,
 						     GtkReliefStyle  newstyle);
@@ -125,14 +131,14 @@ void                  gtk_button_set_image_position (GtkButton      *button,
 						     GtkPositionType position);
 GtkPositionType       gtk_button_get_image_position (GtkButton      *button);
 
-void _gtk_button_set_depressed             (GtkButton     *button,
-					    gboolean       depressed);
-void _gtk_button_paint                     (GtkButton     *button,
-					    GdkRectangle  *area,
-					    GtkStateType   state_type,
-					    GtkShadowType  shadow_type,
-					    const gchar   *main_detail,
-					    const gchar   *default_detail);
+void _gtk_button_set_depressed             (GtkButton          *button,
+					    gboolean            depressed);
+void _gtk_button_paint                     (GtkButton          *button,
+					    const GdkRectangle *area,
+					    GtkStateType        state_type,
+					    GtkShadowType       shadow_type,
+					    const gchar        *main_detail,
+					    const gchar        *default_detail);
 
 G_END_DECLS
 
