@@ -32,7 +32,6 @@
 #define __GTK_FIXED_H__
 
 
-#include <gdk/gdk.h>
 #include <gtk/gtkcontainer.h>
 
 
@@ -80,9 +79,11 @@ void       gtk_fixed_move              (GtkFixed       *fixed,
                                         GtkWidget      *widget,
                                         gint            x,
                                         gint            y);
+#ifndef GTK_DISABLE_DEPRECATED
 void       gtk_fixed_set_has_window    (GtkFixed       *fixed,
 					gboolean        has_window);
 gboolean   gtk_fixed_get_has_window    (GtkFixed       *fixed);
+#endif
 
 G_END_DECLS
 
